@@ -3,14 +3,13 @@
 {-# LANGUAGE GADTs                      #-}
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE MultiParamTypeClasses      #-}
-{-# LANGUAGE OverlappingInstances       #-}
-{-# LANGUAGE TemplateHaskell            #-}
 {-# LANGUAGE TypeFamilies               #-}
 {-# LANGUAGE TypeSynonymInstances       #-}
+
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Diagrams.Core.Names
--- Copyright   :  (c) 2011 diagrams-core team (see LICENSE)
+-- Copyright   :  (c) 2011-2015 diagrams-core team (see LICENSE)
 -- License     :  BSD-style (see LICENSE)
 -- Maintainer  :  diagrams-discuss@googlegroups.com
 --
@@ -20,26 +19,27 @@
 -----------------------------------------------------------------------------
 
 module Diagrams.Core.Names
-       (-- * Names
-        -- ** Atomic names
-         AName(..)
+  (-- * Names
+   -- ** Atomic names
+    AName(..)
+  , _AName
 
-        -- ** Names
-       , Name(..), IsName(..), (.>)
+   -- ** Names
+  , Name(..), IsName(..), (.>)
 
-        -- ** Qualifiable
-       , Qualifiable(..)
+   -- ** Qualifiable
+  , Qualifiable(..)
 
-       ) where
+  ) where
 
-import           Control.Lens            (over, Wrapped(..), Rewrapped, iso, _Unwrapping')
-import           Data.List               (intercalate)
+import           Control.Lens            hiding ((.>))
 import qualified Data.Map                as M
 import           Data.Semigroup
 import qualified Data.Set                as S
 import           Data.Typeable
 
 import           Diagrams.Core.Transform
+import           Diagrams.Core.Measure
 
 ------------------------------------------------------------
 --  Names  -------------------------------------------------
@@ -80,8 +80,8 @@ instance IsName Int
 instance IsName Float
 instance IsName Double
 instance IsName Integer
-instance IsName String
 instance IsName a => IsName [a]
+instance IsName a => IsName (Maybe a)
 instance (IsName a, IsName b) => IsName (a,b)
 instance (IsName a, IsName b, IsName c) => IsName (a,b,c)
 
@@ -89,7 +89,7 @@ instance (IsName a, IsName b, IsName c) => IsName (a,b,c)
 --   things which are 'Typeable', 'Ord' and 'Show'.
 data AName where
   AName :: (Typeable a, Ord a, Show a) => a -> AName
-  deriving (Typeable)
+  deriving Typeable
 
 instance IsName AName where
   toName = Name . (:[])
@@ -103,24 +103,35 @@ instance Eq AName where
 instance Ord AName where
   (AName a1) `compare` (AName a2) =
     case cast a2 of
-      Nothing  -> show (typeOf a1) `compare` show (typeOf a2)
       Just a2' -> a1 `compare` a2'
+      Nothing  -> typeOf a1 `compare` typeOf a2
 
 instance Show AName where
-  show (AName a) = show a
+  showsPrec d (AName a) = showParen (d > 10) $
+    showString "AName " . showsPrec 11 a
+
+-- | Prism onto 'AName'.
+_AName :: (Typeable a, Ord a, Show a) => Prism' AName a
+_AName = prism' AName (\(AName a) -> cast a)
 
 -- | A (qualified) name is a (possibly empty) sequence of atomic names.
 newtype Name = Name [AName]
   deriving (Eq, Ord, Semigroup, Monoid, Typeable)
 
-instance Wrapped Name where
-    type Unwrapped Name = [AName]
-    _Wrapped' = iso (\(Name ans) -> ans) Name
-
 instance Rewrapped Name Name
+instance Wrapped Name where
+  type Unwrapped Name = [AName]
+  _Wrapped' = iso (\(Name ns) -> ns) Name
 
 instance Show Name where
-  show (Name ns) = intercalate " .> " $ map show ns
+  showsPrec d (Name xs) = case xs of
+    []     -> showParen (d > 10) $ showString "Name []"
+    [n]    -> showParen (d > 10) $ showString "toName " . showsName 11 n
+    (n:ns) -> showParen (d > 5)  $ showsName 6 n . go ns
+      where
+        go (y:ys) = showString " .> " . showsName 6 y . go ys
+        go _      = id
+    where showsName dd (AName a) = showsPrec dd a
 
 instance IsName Name where
   toName = id
@@ -135,32 +146,35 @@ a1 .> a2 = toName a1 <> toName a2
 --   prefixing them with a name.
 class Qualifiable q where
   -- | Qualify with the given name.
-  (|>) :: IsName a => a -> q -> q
+  (.>>) :: IsName a => a -> q -> q
 
 -- | Of course, names can be qualified using @(.>)@.
 instance Qualifiable Name where
-  (|>) = (.>)
+  (.>>) = (.>)
 
 instance Qualifiable a => Qualifiable (TransInv a) where
-  (|>) n = over (_Unwrapping' TransInv) (n |>)
+  (.>>) n = over (_Unwrapping' TransInv) (n .>>)
 
 instance (Qualifiable a, Qualifiable b) => Qualifiable (a,b) where
-  n |> (a,b) = (n |> a, n |> b)
+  n .>> (a,b) = (n .>> a, n .>> b)
 
 instance (Qualifiable a, Qualifiable b, Qualifiable c) => Qualifiable (a,b,c) where
-  n |> (a,b,c) = (n |> a, n |> b, n |> c)
+  n .>> (a,b,c) = (n .>> a, n .>> b, n .>> c)
 
 instance Qualifiable a => Qualifiable [a] where
-  n |> as = map (n |>) as
+  n .>> as = map (n .>>) as
 
 instance (Ord a, Qualifiable a) => Qualifiable (S.Set a) where
-  n |> s = S.map (n |>) s
+  n .>> s = S.map (n .>>) s
 
 instance Qualifiable a => Qualifiable (M.Map k a) where
-  n |> m = fmap (n |>) m
+  n .>> m = fmap (n .>>) m
 
 instance Qualifiable a => Qualifiable (b -> a) where
- n |> f = (n |>) . f
+  n .>> f = (n .>>) . f
 
-infixr 5 |>
+instance Qualifiable a => Qualifiable (Measured n a) where
+  n .>> m = fmap (n .>>) m
+
+infixr 5 .>>
 infixr 5 .>
